@@ -43,22 +43,25 @@
 #include <config.h>
 #endif
 
-#include <linux/version.h>
 #include <linux/input.h>
 
 #include <string.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <dirent.h>
 #include <errno.h>
 #include <getopt.h>
 #include <ctype.h>
 #include <signal.h>
+#include <limits.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <unistd.h>
+
+#ifndef input_event_sec
+#define input_event_sec time.tv_sec
+#define input_event_usec time.tv_usec
+#endif
 
 #define BITS_PER_LONG (sizeof(long) * 8)
 #define NBITS(x) ((((x)-1)/BITS_PER_LONG)+1)
@@ -105,6 +108,7 @@ static const struct query_mode {
 };
 
 static int grab_flag = 0;
+static char* progname;
 static volatile sig_atomic_t stop = 0;
 
 static void interrupt_handler(int sig)
@@ -522,6 +526,9 @@ static const char * const keys[KEY_MAX + 1] = {
 	NAME_ELEMENT(BTN_TOOL_BRUSH),		NAME_ELEMENT(BTN_TOOL_PENCIL),
 	NAME_ELEMENT(BTN_TOOL_AIRBRUSH),	NAME_ELEMENT(BTN_TOOL_FINGER),
 	NAME_ELEMENT(BTN_TOOL_MOUSE),		NAME_ELEMENT(BTN_TOOL_LENS),
+#ifdef BTN_STYLUS3
+	NAME_ELEMENT(BTN_STYLUS3),
+#endif
 	NAME_ELEMENT(BTN_TOUCH),		NAME_ELEMENT(BTN_STYLUS),
 	NAME_ELEMENT(BTN_STYLUS2),		NAME_ELEMENT(BTN_TOOL_DOUBLETAP),
 	NAME_ELEMENT(BTN_TOOL_TRIPLETAP),
@@ -875,7 +882,7 @@ static char* scan_devices(void)
 	char *filename;
 	int max_device = 0;
 
-	ndev = scandir(DEV_INPUT_EVENT, &namelist, is_event_device, versionsort);
+	ndev = scandir(DEV_INPUT_EVENT, &namelist, is_event_device, alphasort);
 	if (ndev <= 0)
 		return NULL;
 
@@ -883,7 +890,7 @@ static char* scan_devices(void)
 
 	for (i = 0; i < ndev; i++)
 	{
-		char fname[PATH_MAX];
+		char fname[4096];
 		int fd = -1;
 		char name[256] = "???";
 
@@ -923,7 +930,7 @@ static int version(void)
 #ifndef PACKAGE_VERSION
 #define PACKAGE_VERSION "<version undefined>"
 #endif
-	printf("%s %s\n", program_invocation_short_name, PACKAGE_VERSION);
+	printf("%s %s\n", progname, PACKAGE_VERSION);
 	return EXIT_SUCCESS;
 }
 
@@ -935,12 +942,11 @@ static int usage(void)
 {
 	printf("USAGE:\n");
 	printf(" Capture mode:\n");
-	printf("   %s [--grab] /dev/input/eventX\n", program_invocation_short_name);
+	printf("   %s [--grab] /dev/input/eventX\n", progname);
 	printf("     --grab  grab the device for exclusive access\n");
 	printf("\n");
 	printf(" Query mode: (check exit code)\n");
-	printf("   %s --query /dev/input/eventX <type> <value>\n",
-		program_invocation_short_name);
+	printf("   %s --query /dev/input/eventX <type> <value>\n", progname);
 
 	printf("\n");
 	printf("<type> is one of: EV_KEY, EV_SW, EV_LED, EV_SND\n");
@@ -1139,7 +1145,7 @@ static int print_events(int fd)
 			type = ev[i].type;
 			code = ev[i].code;
 
-			printf("Event: time %ld.%06ld, ", ev[i].time.tv_sec, ev[i].time.tv_usec);
+			printf("Event: time %ld.%06ld, ", ev[i].input_event_sec, ev[i].input_event_usec);
 
 			if (type == EV_SYN) {
 				if (code == SYN_MT_REPORT)
@@ -1242,8 +1248,10 @@ static int do_capture(const char *device, int grab_flag)
 		printf("***********************************************\n");
 	}
 
-	signal(SIGINT, interrupt_handler);
-	signal(SIGTERM, interrupt_handler);
+	if (grab_flag) {
+		signal(SIGINT, interrupt_handler);
+		signal(SIGTERM, interrupt_handler);
+	}
 
 	free(filename);
 
@@ -1340,6 +1348,8 @@ int main (int argc, char **argv)
 	const char *event_type;
 	enum evtest_mode mode = MODE_CAPTURE;
 
+	progname = argv[0];
+
 	while (1) {
 		int option_index = 0;
 		int c = getopt_long(argc, argv, "", long_options, &option_index);
